@@ -51,6 +51,17 @@ def to_valid_python_name(spec_name: str) -> str:
     return sanitized
 
 
+def html_to_rst(html: str):
+    import pypandoc
+
+    doc = pypandoc.convert_text(html, "rst", format="html")
+    doc = doc.replace("\_", "_")  # noqa: W605
+    doc = doc.replace("\|", "|")  # noqa: W605
+    doc = doc.replace("\ ", " ")  # noqa: W605
+    rst = doc.strip()
+    return rst
+
+
 class ShapeNode:
     service: ServiceModel
     shape: Shape
@@ -132,9 +143,15 @@ class ShapeNode:
 
         for k, v in self.shape.members.items():
             if k in self.shape.required_members:
-                output.write(f"    {k}: {q}{to_valid_python_name(v.name)}{q}\n")
+                if v.serialization.get("eventstream"):
+                    output.write(f"    {k}: Iterator[{q}{to_valid_python_name(v.name)}{q}]\n")
+                else:
+                    output.write(f"    {k}: {q}{to_valid_python_name(v.name)}{q}\n")
             else:
-                output.write(f"    {k}: Optional[{q}{to_valid_python_name(v.name)}{q}]\n")
+                if v.serialization.get("eventstream"):
+                    output.write(f"    {k}: Iterator[{q}{to_valid_python_name(v.name)}{q}]\n")
+                else:
+                    output.write(f"    {k}: Optional[{q}{to_valid_python_name(v.name)}{q}]\n")
 
     def _print_as_typed_dict(self, output, doc=True, quote_types=False):
         name = to_valid_python_name(self.shape.name)
@@ -142,20 +159,23 @@ class ShapeNode:
         output.write('%s = TypedDict("%s", {\n' % (name, name))
         for k, v in self.shape.members.items():
             if k in self.shape.required_members:
-                output.write(f'    "{k}": {q}{to_valid_python_name(v.name)}{q},\n')
+                if v.serialization.get("eventstream"):
+                    output.write(f'    "{k}": Iterator[{q}{to_valid_python_name(v.name)}{q}],\n')
+                else:
+                    output.write(f'    "{k}": {q}{to_valid_python_name(v.name)}{q},\n')
             else:
-                output.write(f'    "{k}": Optional[{q}{to_valid_python_name(v.name)}{q}],\n')
+                if v.serialization.get("eventstream"):
+                    output.write(f'    "{k}": Iterator[{q}{to_valid_python_name(v.name)}{q}],\n')
+                else:
+                    output.write(f'    "{k}": Optional[{q}{to_valid_python_name(v.name)}{q}],\n')
         output.write("}, total=False)")
 
     def print_shape_doc(self, output, shape):
         html = shape.documentation
-        import pypandoc
-
-        doc = pypandoc.convert_text(html, "rst", format="html")
-        rst = doc.strip()
+        rst = html_to_rst(html)
         if rst:
             output.write('    """')
-            output.write(f"{doc.strip()}\n")
+            output.write(f"{rst}\n")
             output.write('    """\n')
 
     def print_declaration(self, output, doc=True, quote_types=False):
@@ -226,7 +246,7 @@ class ShapeNode:
 
 def generate_service_types(output, service: ServiceModel, doc=True):
     output.write("import sys\n")
-    output.write("from typing import Dict, List, Optional\n")
+    output.write("from typing import Dict, List, Optional, Iterator\n")
     output.write("from datetime import datetime\n")
     output.write("if sys.version_info >= (3, 8):\n")
     output.write("    from typing import TypedDict\n")
@@ -329,19 +349,17 @@ def generate_service_api(output, service: ServiceModel, doc=True):
         # convert html documentation to rst and print it into to the signature
         if doc:
             html = operation.documentation
-            import pypandoc
-
-            doc = pypandoc.convert_text(html, "rst", format="html")
+            rst = html_to_rst(html)
             output.write('        """')
-            output.write(f"{doc.strip()}\n")
+            output.write(f"{rst}\n")
             output.write("\n")
 
             # parameters
             for param_name, shape in param_shapes.items():
                 # FIXME: this doesn't work properly
-                pdoc = pypandoc.convert_text(shape.documentation, "rst", format="html")
-                pdoc = pdoc.strip().split(".")[0] + "."
-                output.write(f":param {param_name}: {pdoc}\n")
+                rst = html_to_rst(shape.documentation)
+                rst = rst.strip().split(".")[0] + "."
+                output.write(f":param {param_name}: {rst}\n")
 
             # return value
             if operation.output_shape:
@@ -408,12 +426,12 @@ def generate_code(service_name: str, doc: bool = False) -> str:
         import isort
         from black import FileMode, format_str
 
-        # try to format with black
-        code = format_str(code, mode=FileMode(line_length=100))
-
         # try to remove unused imports
         code = autoflake.fix_code(code, remove_all_unused_imports=True)
 
+        # try to format with black
+        code = format_str(code, mode=FileMode(line_length=100))
+
         # try to sort imports
         code = isort.code(code, config=isort.Config(profile="black", line_length=100))
     except Exception:
