@@ -23,6 +23,7 @@ from localstack.services.awslambda.lambda_utils import (
     LAMBDA_RUNTIME_PYTHON36,
     LAMBDA_RUNTIME_PYTHON37,
     LAMBDA_RUNTIME_PYTHON38,
+    get_executor_mode,
     is_python_runtime,
 )
 from localstack.utils import common
@@ -115,7 +116,7 @@ def _init_java_agent_configs():
         "https://repo.thundra.io/service/local/artifact/maven/redirect?"
         + "r=thundra-releases&g=io.thundra.agent&a=thundra-agent-lambda-bootstrap&v={v}"
     ).format(v=version)
-    THUNDRA_JAVA_AGENT_LOCAL_PATH = "%s/%s" % (config.TMP_FOLDER, jar_name)
+    THUNDRA_JAVA_AGENT_LOCAL_PATH = "%s/%s" % (config.dirs.tmp, jar_name)
 
 
 def _install_java_agent():
@@ -217,11 +218,11 @@ def _init_node_agent_configs() -> bool:
 
     THUNDRA_NODE_AGENT_VERSION = version.strip()
     THUNDRA_NODE_AGENT_LOCAL_PATH = "%s/thundra/node/%s/" % (
-        config.TMP_FOLDER,
+        config.dirs.tmp,
         THUNDRA_NODE_AGENT_VERSION,
     )
     THUNDRA_NODE_AGENT_LOCAL_PATH_ON_HOST = "%s/thundra/node/%s/" % (
-        config.HOST_TMP_FOLDER,
+        config.dirs.functions,
         THUNDRA_NODE_AGENT_VERSION,
     )
 
@@ -327,11 +328,11 @@ def _init_python_agent_configs() -> bool:
 
     THUNDRA_PYTHON_AGENT_VERSION = version.strip()
     THUNDRA_PYTHON_AGENT_LOCAL_PATH = "%s/thundra/python/%s/" % (
-        config.TMP_FOLDER,
+        config.dirs.tmp,
         THUNDRA_PYTHON_AGENT_VERSION,
     )
     THUNDRA_PYTHON_AGENT_LOCAL_PATH_ON_HOST = "%s/thundra/python/%s/" % (
-        config.HOST_TMP_FOLDER,
+        config.dirs.functions,
         THUNDRA_PYTHON_AGENT_VERSION,
     )
 
@@ -406,11 +407,11 @@ def _prepare_invocation_for_python_lambda(
 class LambdaExecutorPluginThundra(LambdaExecutorPlugin):
     def should_apply(self, context: InvocationContext) -> bool:
         # Local executor is not supported yet
-        if "local" in config.LAMBDA_EXECUTOR:
+        if "local" in get_executor_mode():
             return False
 
-        # Plugin can only applied if LAMBDA_REMOTE_DOCKER=0
-        if "docker" in config.LAMBDA_EXECUTOR and config.LAMBDA_REMOTE_DOCKER:
+        # Plugin can only be applied if LAMBDA_REMOTE_DOCKER=0
+        if "docker" in get_executor_mode() and config.LAMBDA_REMOTE_DOCKER:
             return False
 
         # Plugin can only applied if API key is configured
