@@ -164,31 +164,11 @@ class EventForwarder:
             forward_records = cls.prepare_records_to_forward_to_ddb_stream(records)
             records_to_ddb = copy.deepcopy(forward_records)
             cls.forward_to_ddb_stream(records_to_ddb)
-            # lambda receives the same records as the ddb streams
-            cls.forward_to_lambda(forward_records)
 
         if background:
             return start_worker_thread(_forward)
         _forward()
 
-    @staticmethod
-    def forward_to_lambda(records):
-        for record in records:
-            event_source_arn = record.get("eventSourceARN")
-            if not event_source_arn:
-                continue
-            sources = lambda_api.get_event_sources(source_arn=event_source_arn)
-            event = {"Records": [record]}
-            for src in sources:
-                if src.get("State") != "Enabled":
-                    continue
-                lambda_api.run_lambda(
-                    func_arn=src["FunctionArn"],
-                    event=event,
-                    context={},
-                    asynchronous=not config.SYNCHRONOUS_DYNAMODB_EVENTS,
-                )
-
     @staticmethod
     def forward_to_ddb_stream(records):
         dynamodbstreams_api.forward_events(records)
@@ -223,7 +203,6 @@ class EventForwarder:
         # StreamViewType determines what information is written to the stream for the table
         # When an item in the table is inserted, updated or deleted
         for record in records:
-            record.pop("eventID", None)
             ddb_record = record["dynamodb"]
             stream_type = ddb_record.get("StreamViewType")
             if not stream_type:
@@ -304,27 +283,16 @@ class DynamoDBApiListener(AwsApiListener):
         self.provider = provider
         super().__init__("dynamodb", HttpFallbackDispatcher(provider, provider.get_forward_url))
 
-    def forward_request(self, method, path, data, headers):
-        action = headers.get("X-Amz-Target", "")
-        action = action.replace(ACTION_PREFIX, "")
-        if self.provider.should_throttle(action):
-            message = (
-                "The level of configured provisioned throughput for the table was exceeded. "
-                + "Consider increasing your provisioning level with the UpdateTable API"
-            )
-            raise ProvisionedThroughputExceededException(message)
-
-        return super().forward_request(method, path, data, headers)
-
     def return_response(self, method, path, data, headers, response):
         if response._content:
+            response_content = to_str(response._content)
             # fix the table and latest stream ARNs (DynamoDBLocal hardcodes "ddblocal" as the region)
             content_replaced = re.sub(
-                r'("TableArn"|"LatestStreamArn"|"StreamArn")\s*:\s*"arn:aws:dynamodb:ddblocal:([^"]+)"',
-                rf'\1: "arn:aws:dynamodb:{aws_stack.get_region()}:\2"',
-                to_str(response._content),
+                r'("TableArn"|"LatestStreamArn"|"StreamArn")\s*:\s*"arn:([a-z-]+):dynamodb:ddblocal:([^"]+)"',
+                rf'\1: "arn:\2:dynamodb:{aws_stack.get_region()}:\3"',
+                response_content,
             )
-            if content_replaced != response._content:
+            if content_replaced != response_content:
                 response._content = content_replaced
 
         # set x-amz-crc32 headers required by some client
@@ -355,6 +323,9 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
     def forward_request(
         self, context: RequestContext, service_request: ServiceRequest = None
     ) -> ServiceResponse:
+        # check rate limiting for this request and raise an error, if provisioned throughput is exceeded
+        self.check_provisioned_throughput(context.operation.name)
+
         # note: modifying headers in-place here before forwarding the request
         self.prepare_request_headers(context.request.headers)
         return self.request_forwarder(context, service_request)
@@ -1040,7 +1011,7 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
         if not self.table_exists(table_name):
             raise ResourceNotFoundException("Cannot do operations on a non-existent table")
 
-        table_def = DynamoDBRegion.get().table_definitions.get(table_name)
+        table_def = DynamoDBRegion.get().table_definitions.get(table_name) or {}
 
         stream_destinations = table_def.get("KinesisDataStreamDestinations") or []
         return DescribeKinesisStreamingDestinationOutput(
@@ -1197,7 +1168,7 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
                         )
                         new_record = copy.deepcopy(record)
                         new_record["eventID"] = short_uid()
-                        new_record["dynamodb"]["SizeBytes"] = len(json.dumps(put_request["Item"]))
+                        new_record["dynamodb"]["SizeBytes"] = _get_size_bytes(put_request["Item"])
                         new_record["eventName"] = "INSERT" if not existing_item else "MODIFY"
                         new_record["dynamodb"]["Keys"] = keys
                         new_record["dynamodb"]["NewImage"] = put_request["Item"]
@@ -1220,7 +1191,7 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
                         new_record["eventName"] = "REMOVE"
                         new_record["dynamodb"]["Keys"] = keys
                         new_record["dynamodb"]["OldImage"] = existing_items[i]
-                        new_record["dynamodb"]["SizeBytes"] = len(json.dumps(existing_items[i]))
+                        new_record["dynamodb"]["SizeBytes"] = _get_size_bytes(existing_items[i])
                         new_record["eventSourceARN"] = aws_stack.dynamodb_table_arn(table_name)
                         records.append(new_record)
                     if unprocessed_delete_items and len(unprocessed_delete_items) > i:
@@ -1265,6 +1236,14 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
             "eventSource": "aws:dynamodb",
         }
 
+    def check_provisioned_throughput(self, action):
+        if self.should_throttle(action):
+            message = (
+                "The level of configured provisioned throughput for the table was exceeded. "
+                + "Consider increasing your provisioning level with the UpdateTable API"
+            )
+            raise ProvisionedThroughputExceededException(message)
+
     def action_should_throttle(self, action, actions):
         throttled = [f"{ACTION_PREFIX}{a}" for a in actions]
         return (action in throttled) or (action in actions)
@@ -1283,13 +1262,18 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
             action, THROTTLED_ACTIONS
         ):
             return True
-        else:
-            return False
+        return False
 
 
 # ---
 # Misc. util functions
 # ---
+def _get_size_bytes(item) -> int:
+    try:
+        size_bytes = len(json.dumps(item))
+    except TypeError:
+        size_bytes = len(str(item))
+    return size_bytes
 
 
 def get_global_secondary_index(table_name, index_name):
@@ -1389,7 +1373,7 @@ def get_updated_records(table_name: str, existing_items: List) -> List:
             "dynamodb": {
                 "Keys": keys,
                 "NewImage": new_image,
-                "SizeBytes": len(json.dumps(item)),
+                "SizeBytes": _get_size_bytes(item),
             },
         }
         if stream_spec:
