@@ -133,7 +133,15 @@ class ServiceLifecycleHook:
         pass
 
 
-class Service(object):
+class StateLifecycle:
+    def retrieve_state(self):
+        pass
+
+    def inject_state(self, state):
+        pass
+
+
+class Service:
     def __init__(
         self,
         name,
@@ -143,6 +151,7 @@ class Service(object):
         active=False,
         stop=None,
         lifecycle_hook: ServiceLifecycleHook = None,
+        state_lifecycle: StateLifecycle = None,
     ):
         self.plugin_name = name
         self.start_function = start
@@ -151,6 +160,7 @@ class Service(object):
         self.default_active = active
         self.stop_function = stop
         self.lifecycle_hook = lifecycle_hook or ServiceLifecycleHook()
+        self.state_lifecycle = state_lifecycle
         call_safe(self.lifecycle_hook.on_after_init)
 
     def start(self, asynchronous):
@@ -485,6 +495,17 @@ class ServicePluginManager(ServiceManager):
             if self.get_active_provider(service) in providers
         ]
 
+    def list_loaded_services(self) -> List[str]:
+        """
+        Lists all the services which have a provider that has been initialized
+        :return: a list of service names
+        """
+        return [
+            service
+            for service in self.list_available()
+            if super(ServicePluginManager, self).get_service_container(service)
+        ]
+
     def exists(self, name: str) -> bool:
         return name in self.list_available()
 
