@@ -3,6 +3,7 @@ import logging
 import re
 import time
 from collections import namedtuple
+from typing import Dict
 from urllib import parse as urlparse
 from urllib.parse import parse_qs, urlencode
 
@@ -86,9 +87,9 @@ def is_static_website(headers):
     return bool(re.match(S3_STATIC_WEBSITE_HOST_REGEX, headers.get("host", "")))
 
 
-def uses_host_addressing(headers):
+def uses_host_addressing(headers: Dict[str, str]):
     """
-    Determines if the bucket is using host based addressing style or path based
+    Determines if the bucket is using host based addressing style or path based.
     """
     # we can assume that the host header we are receiving here is actually the header we originally received
     # from the client (because the edge service is forwarding the request in memory)
@@ -112,7 +113,7 @@ def extract_bucket_name(headers, path):
         if match and match.group(3):
             bucket_name = match.group(3)
     else:
-        bucket_name = path.split("/", 2)[1]
+        bucket_name = path.split("/", maxsplit=2)[1]
     return bucket_name if bucket_name else None
 
 
@@ -123,11 +124,11 @@ def extract_key_name(headers, path):
     key_name = None
     path = path.split("?")[0]  # strip off query params from path
     if uses_host_addressing(headers):
-        split = path.split("/", 1)
+        split = path.split("/", maxsplit=1)
         if len(split) > 1:
             key_name = split[1]
     else:
-        split = path.split("/", 2)
+        split = path.split("/", maxsplit=2)
         if len(split) > 2:
             key_name = split[2]
 
@@ -184,6 +185,11 @@ def get_key_from_s3_url(url: str, leading_slash: bool = False) -> str:
     return result
 
 
+def is_object_download_request(method, path, headers) -> bool:
+    """Return whether this is a GetObject download request."""
+    return method == "GET" and bool(extract_key_name(headers, path))
+
+
 def is_expired(expiry_datetime):
     now_datetime = datetime.datetime.now(tz=expiry_datetime.tzinfo)
     return now_datetime > expiry_datetime
