@@ -16,7 +16,7 @@ from typing import Dict, List, NamedTuple, Optional, Set
 from moto.sqs.models import BINARY_TYPE_FIELD_INDEX, STRING_TYPE_FIELD_INDEX
 from moto.sqs.models import Message as MotoMessage
 
-from localstack import config
+from localstack import config, constants
 from localstack.aws.api import CommonServiceException, RequestContext
 from localstack.aws.api.sqs import (
     ActionNameList,
@@ -49,7 +49,6 @@ from localstack.aws.api.sqs import (
     QueueAttributeMap,
     QueueAttributeName,
     QueueDoesNotExist,
-    QueueNameExists,
     ReceiptHandleIsInvalid,
     ReceiveMessageResult,
     SendMessageBatchRequestEntryList,
@@ -64,6 +63,7 @@ from localstack.aws.api.sqs import (
 )
 from localstack.aws.spec import load_service
 from localstack.config import external_service_url
+from localstack.services.generic_proxy import RegionBackend
 from localstack.services.plugins import ServiceLifecycleHook
 from localstack.utils.aws.aws_stack import parse_arn
 from localstack.utils.common import long_uid, md5, now, start_thread
@@ -85,18 +85,9 @@ FIFO_MSG_REGEX = "^[0-9a-zA-z!\"#$%&'()*+,./:;<=>?@[\\]^_`{|}~-]*$"
 DEDUPLICATION_INTERVAL_IN_SEC = 5 * 60
 
 
-def generate_message_id():
-    return long_uid()
-
-
-def generate_receipt_handle():
-    # http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/ImportantIdentifiers.html#ImportantIdentifiers-receipt-handles
-    return "".join(random.choices(string.ascii_letters + string.digits, k=172)) + "="
-
-
 class InvalidParameterValue(CommonServiceException):
     def __init__(self, message):
-        super().__init__("InvalidParameterValues", message, 400, True)
+        super().__init__("InvalidParameterValue", message, 400, True)
 
 
 class InvalidAttributeValue(CommonServiceException):
@@ -109,6 +100,10 @@ class MissingParameter(CommonServiceException):
         super().__init__("MissingParameter", message, 400, True)
 
 
+def generate_message_id():
+    return long_uid()
+
+
 def assert_queue_name(queue_name: str, fifo: bool = False):
     if queue_name.endswith(".fifo"):
         if not fifo:
@@ -133,10 +128,25 @@ def check_message_content(message_body: str):
         raise InvalidMessageContents(error)
 
 
-class QueueKey(NamedTuple):
-    region: str
-    account_id: str
-    name: str
+def encode_receipt_handle(queue_arn, message: "SqsMessage") -> str:
+    # http://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/ImportantIdentifiers.html#ImportantIdentifiers-receipt-handles
+    # encode the queue arn in the receipt handle, so we can later check if it belongs to the queue
+    # but also add some randomness s.t. the generated receipt handles look like the ones from AWS
+    handle = f"{long_uid()} {queue_arn} {message.message.get('MessageId')} {message.last_received}"
+    encoded = base64.b64encode(handle.encode("utf-8"))
+    return encoded.decode("utf-8")
+
+
+def decode_receipt_handle(receipt_handle: str) -> str:
+    try:
+        handle = base64.b64decode(receipt_handle).decode("utf-8")
+        _, queue_arn, message_id, last_received = handle.split(" ")
+        parse_arn(queue_arn)  # raises a ValueError if it is not an arn
+        return queue_arn
+    except (IndexError, ValueError):
+        raise ReceiptHandleIsInvalid(
+            f'The input receipt handle "{receipt_handle}" is not a valid receipt handle.'
+        )
 
 
 class Permission(NamedTuple):
@@ -171,8 +181,25 @@ class SqsMessage:
         self.first_received = None
         self.deleted = False
         self.priority = priority
-        self.message_deduplication_id = message_deduplication_id
-        self.message_group_id = message_group_id
+
+        attributes = {}
+        if message_group_id is not None:
+            attributes["MessageGroupId"] = message_group_id
+        if message_deduplication_id is not None:
+            attributes["MessageDeduplicationId"] = message_deduplication_id
+
+        if self.message.get("Attributes"):
+            self.message["Attributes"].update(attributes)
+        else:
+            self.message["Attributes"] = attributes
+
+    @property
+    def message_group_id(self) -> Optional[str]:
+        return self.message["Attributes"].get("MessageGroupId")
+
+    @property
+    def message_deduplication_id(self) -> Optional[str]:
+        return self.message["Attributes"].get("MessageDeduplicationId")
 
     @property
     def is_visible(self):
@@ -203,7 +230,9 @@ class SqsMessage:
 
 
 class SqsQueue:
-    key: QueueKey
+    name: str
+    region: str
+    account_id: str
 
     attributes: QueueAttributeMap
     tags: TagMap
@@ -215,10 +244,12 @@ class SqsQueue:
     inflight: Set[SqsMessage]
     receipts: Dict[str, SqsMessage]
 
-    def __init__(self, key: QueueKey, attributes=None, tags=None) -> None:
-        super().__init__()
-        self._assert_queue_name(key.name)
-        self.key = key
+    def __init__(self, name: str, region: str, account_id: str, attributes=None, tags=None) -> None:
+        self.name = name
+        self.region = region
+        self.account_id = account_id
+
+        self._assert_queue_name(name)
         self.tags = tags or {}
 
         self.visible = PriorityQueue()
@@ -235,17 +266,17 @@ class SqsQueue:
 
     def default_attributes(self) -> QueueAttributeMap:
         return {
-            QueueAttributeName.QueueArn: self.arn,
             QueueAttributeName.ApproximateNumberOfMessages: self.visible._qsize,
             QueueAttributeName.ApproximateNumberOfMessagesNotVisible: lambda: len(self.inflight),
             QueueAttributeName.ApproximateNumberOfMessagesDelayed: "0",  # FIXME: this should also be callable
             QueueAttributeName.CreatedTimestamp: str(now()),
+            QueueAttributeName.DelaySeconds: "0",
             QueueAttributeName.LastModifiedTimestamp: str(now()),
-            QueueAttributeName.VisibilityTimeout: "30",
             QueueAttributeName.MaximumMessageSize: "262144",
             QueueAttributeName.MessageRetentionPeriod: "345600",
-            QueueAttributeName.DelaySeconds: "0",
+            QueueAttributeName.QueueArn: self.arn,
             QueueAttributeName.ReceiveMessageWaitTimeSeconds: "0",
+            QueueAttributeName.VisibilityTimeout: "30",
         }
 
     def update_last_modified(self, timestamp: int = None):
@@ -254,39 +285,59 @@ class SqsQueue:
 
         self.attributes[QueueAttributeName.LastModifiedTimestamp] = str(timestamp)
 
-    @property
-    def name(self):
-        return self.key.name
-
-    @property
-    def owner(self):
-        return self.key.account_id
-
     @property
     def arn(self) -> str:
-        return f"arn:aws:sqs:{self.key.region}:{self.key.account_id}:{self.key.name}"
+        return f"arn:aws:sqs:{self.region}:{self.account_id}:{self.name}"
 
     def url(self, context: RequestContext) -> str:
-        """Return queue URL using either SQS_PORT_EXTERNAL (if configured), or based on the 'Host' request header"""
+        """Return queue URL using either SQS_PORT_EXTERNAL (if configured), the SQS_ENDPOINT_STRATEGY (if configured)
+        or based on the 'Host' request header"""
+
         host_url = context.request.host_url
-        if config.SQS_PORT_EXTERNAL:
-            host_url = external_service_url("sqs")
+
+        if config.SQS_ENDPOINT_STRATEGY == "domain":
+            # queue.localhost.localstack.cloud:4566/000000000000/my-queue (us-east-1)
+            # or us-east-2.queue.localhost.localstack.cloud:4566/000000000000/my-queue
+            region = "" if self.region == "us-east-1" else self.region + "."
+            scheme = context.request.scheme
+            host_url = f"{scheme}://{region}queue.{constants.LOCALHOST_HOSTNAME}:{config.EDGE_PORT}"
+        elif config.SQS_ENDPOINT_STRATEGY == "path":
+            # localhost:4566/queue/us-east-1/00000000000/my-queue (us-east-1)
+            host_url = f"{context.request.host}/queue/{self.region}"
+        else:
+            if config.SQS_PORT_EXTERNAL:
+                host_url = external_service_url("sqs")
+
         return "{host}/{account_id}/{name}".format(
             host=host_url.rstrip("/"),
-            account_id=self.key.account_id,
-            name=self.key.name,
+            account_id=self.account_id,
+            name=self.name,
         )
 
     @property
     def visibility_timeout(self) -> int:
         return int(self.attributes[QueueAttributeName.VisibilityTimeout])
 
+    @property
+    def wait_time_seconds(self) -> int:
+        return int(self.attributes[QueueAttributeName.ReceiveMessageWaitTimeSeconds])
+
+    def validate_receipt_handle(self, receipt_handle: str):
+        if self.arn != decode_receipt_handle(receipt_handle):
+            raise ReceiptHandleIsInvalid(
+                f'The input receipt handle "{receipt_handle}" is not a valid receipt handle.'
+            )
+
     def update_visibility_timeout(self, receipt_handle: str, visibility_timeout: int):
         with self.mutex:
+            self.validate_receipt_handle(receipt_handle)
+
             if receipt_handle not in self.receipts:
-                raise ReceiptHandleIsInvalid(
-                    f'The input receipt handle "{receipt_handle}" is not a valid receipt handle.'
+                raise InvalidParameterValue(
+                    f"Value {receipt_handle} for parameter ReceiptHandle is invalid. Reason: Message does not exist "
+                    f"or is not available for visibility timeout change."
                 )
+
             standard_message = self.receipts[receipt_handle]
 
             if standard_message not in self.inflight:
@@ -306,6 +357,8 @@ class SqsQueue:
 
     def remove(self, receipt_handle: str):
         with self.mutex:
+            self.validate_receipt_handle(receipt_handle)
+
             if receipt_handle not in self.receipts:
                 LOG.debug(
                     "no in-flight message found for receipt handle %s in queue %s",
@@ -317,7 +370,9 @@ class SqsQueue:
             standard_message = self.receipts[receipt_handle]
             standard_message.deleted = True
             LOG.debug(
-                "deleting message %s from queue %s", standard_message.message["MessageId"], self.arn
+                "deleting message %s from queue %s",
+                standard_message.message["MessageId"],
+                self.arn,
             )
 
             # remove all handles
@@ -371,7 +426,7 @@ class SqsQueue:
                     standard_message.first_received = standard_message.last_received
 
                 # create and manage receipt handle
-                receipt_handle = generate_receipt_handle()
+                receipt_handle = self.create_receipt_handle(standard_message)
                 standard_message.receipt_handles.add(receipt_handle)
                 self.receipts[receipt_handle] = standard_message
 
@@ -380,13 +435,21 @@ class SqsQueue:
                 else:
                     self.inflight.add(standard_message)
 
-                # prepare message for receiver
-                # TODO: update message attributes (ApproximateFirstReceiveTimestamp, ApproximateReceiveCount)
-                copied_message = copy.deepcopy(standard_message)
-                copied_message.message["ReceiptHandle"] = receipt_handle
+            # prepare message for receiver
+            copied_message = copy.deepcopy(standard_message)
+            copied_message.message["Attributes"][
+                MessageSystemAttributeName.ApproximateReceiveCount
+            ] = str(standard_message.receive_times)
+            copied_message.message["Attributes"][
+                MessageSystemAttributeName.ApproximateFirstReceiveTimestamp
+            ] = standard_message.first_received
+            copied_message.message["ReceiptHandle"] = receipt_handle
 
             return copied_message
 
+    def create_receipt_handle(self, message: SqsMessage) -> str:
+        return encode_receipt_handle(self.arn, message)
+
     def requeue_inflight_messages(self):
         if not self.inflight:
             return
@@ -415,7 +478,7 @@ class SqsQueue:
 
         for k in attributes.keys():
             if k not in valid:
-                raise InvalidAttributeName(f"Unknown attribute name {k}")
+                raise InvalidAttributeName(f"Unknown Attribute {k}")
 
     def generate_sequence_number(self):
         return None
@@ -458,8 +521,8 @@ class FifoQueue(SqsQueue):
     receipts: Dict[str, SqsMessage]
     deduplication: Dict[str, Dict[str, SqsMessage]]
 
-    def __init__(self, key: QueueKey, attributes=None, tags=None) -> None:
-        super().__init__(key, attributes, tags)
+    def __init__(self, name: str, region: str, account_id: str, attributes=None, tags=None) -> None:
+        super().__init__(name, region, account_id, attributes, tags)
         self.deduplication = {}
 
     def put(
@@ -527,7 +590,7 @@ class FifoQueue(SqsQueue):
 
         for k in attributes.keys():
             if k not in valid:
-                raise InvalidAttributeName(f"Unknown attribute name {k}")
+                raise InvalidAttributeName(f"Unknown Attribute {k}")
         # Special Cases
         fifo = attributes.get(QueueAttributeName.FifoQueue)
         if fifo and fifo.lower() != "true":
@@ -548,11 +611,8 @@ class InflightUpdateWorker:
     FIXME: very crude implementation. it would be better to have event-driven communication.
     """
 
-    queues: Dict[QueueKey, SqsQueue]
-
-    def __init__(self, queues: Dict[QueueKey, SqsQueue]) -> None:
+    def __init__(self) -> None:
         super().__init__()
-        self.queues = queues
         self.running = False
         self.thread: Optional[FuncThread] = None
 
@@ -576,8 +636,10 @@ class InflightUpdateWorker:
     def run(self):
         while self.running:
             time.sleep(1)
-            for queue in self.queues.values():
-                queue.requeue_inflight_messages()
+            for region in SqsBackend.regions().keys():
+                backend = SqsBackend.get(region)
+                for queue in backend.queues.values():
+                    queue.requeue_inflight_messages()
 
 
 def check_attributes(message_attributes: MessageBodyAttributeMap):
@@ -616,6 +678,12 @@ def check_attributes(message_attributes: MessageBodyAttributeMap):
         if attribute_type == "String":
             try:
                 attribute_value = attribute.get("StringValue")
+
+                if not attribute_value:
+                    raise InvalidParameterValue(
+                        f"Message (user) attribute '{attribute_name}' must contain a non-empty value of type 'String'."
+                    )
+
                 check_message_content(attribute_value)
             except InvalidMessageContents as e:
                 # AWS throws a different exception here
@@ -636,6 +704,13 @@ def check_fifo_id(fifo_id):
         )
 
 
+class SqsBackend(RegionBackend):
+    queues: Dict[str, SqsQueue]
+
+    def __init__(self):
+        self.queues = {}
+
+
 class SqsProvider(SqsApi, ServiceLifecycleHook):
     """
     LocalStack SQS Provider.
@@ -646,48 +721,39 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         - The region is not encoded in the queue URL
     """
 
-    queues: Dict[QueueKey, SqsQueue]
+    queues: Dict[str, SqsQueue]
 
     def __init__(self) -> None:
         super().__init__()
-        self.queues = {}
         self._mutex = threading.RLock()
-        self._inflight_worker = InflightUpdateWorker(self.queues)
-
-    def start(self):
-        self._inflight_worker.start()
-
-    def shutdown(self):
-        self._inflight_worker.stop()
+        self._inflight_worker = InflightUpdateWorker()
 
     def on_before_start(self):
-        self.start()
+        self._inflight_worker.start()
 
     def on_before_stop(self):
-        self.shutdown()
+        self._inflight_worker.stop()
 
-    def _add_queue(self, queue: SqsQueue):
-        with self._mutex:
-            self.queues[queue.key] = queue
-
-    def _require_queue(self, key: QueueKey) -> SqsQueue:
+    def _require_queue(self, context: RequestContext, name: str) -> SqsQueue:
         """
-        Returns the queue for the given key, or raises QueueDoesNotExist if it does not exist.
+        Returns the queue for the given name, or raises QueueDoesNotExist if it does not exist.
 
-        :param key: the QueueKey to look for
+        :param: context: the request context
+        :param name: the name to look for
         :returns: the queue
         :raises QueueDoesNotExist: if the queue does not exist
         """
+        backend = SqsBackend.get(context.region)
+
         with self._mutex:
-            if key not in self.queues:
-                raise QueueDoesNotExist()
+            if name not in backend.queues.keys():
+                raise QueueDoesNotExist("The specified queue does not exist for this wsdl version.")
 
-            return self.queues[key]
+            return backend.queues[name]
 
-    def _require_queue_by_arn(self, queue_arn: str) -> SqsQueue:
+    def _require_queue_by_arn(self, context: RequestContext, queue_arn: str) -> SqsQueue:
         arn = parse_arn(queue_arn)
-        key = QueueKey(region=arn["region"], account_id=arn["account"], name=arn["resource"])
-        return self._require_queue(key)
+        return self._require_queue(context, arn["resource"])
 
     def _resolve_queue(
         self,
@@ -696,7 +762,7 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         queue_url: Optional[str] = None,
     ) -> SqsQueue:
         """
-        Uses resolve_queue_key to determine the QueueKey from the given input, and returns the respective queue,
+        Determines the name of the queue from available information (request context, queue URL) to return the respective queue,
         or raises QueueDoesNotExist if it does not exist.
 
         :param context: the request context, used for getting region and account_id, and optionally the queue_url
@@ -705,8 +771,8 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         :returns: the queue
         :raises QueueDoesNotExist: if the queue does not exist
         """
-        key = resolve_queue_key(context, queue_name, queue_url)
-        return self._require_queue(key)
+        name = resolve_queue_name(context, queue_name, queue_url)
+        return self._require_queue(context, name)
 
     def create_queue(
         self,
@@ -719,34 +785,36 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
             attributes.get(QueueAttributeName.FifoQueue, "false").lower() == "true"
         )
 
-        k = QueueKey(context.region, context.account_id, queue_name)
-
         # Special Case TODO: why is an emtpy policy passed at all? same in set_queue_attributes
         if attributes and attributes.get(QueueAttributeName.Policy) == "":
             del attributes[QueueAttributeName.Policy]
 
-        if k in self.queues:
-            raise QueueNameExists(queue_name)
+        backend = SqsBackend.get(context.region)
+
+        if queue_name in backend.queues:
+            # FIXME #5938: should raise `QueueNameExists` if queue exists with different attributes
+            queue = backend.queues[queue_name]
+            return CreateQueueResult(QueueUrl=queue.url(context))
         if fifo:
-            queue = FifoQueue(k, attributes, tags)
+            queue = FifoQueue(queue_name, context.region, context.account_id, attributes, tags)
         else:
-            queue = StandardQueue(k, attributes, tags)
-        LOG.debug("creating queue key=%s attributes=%s tags=%s", k, attributes, tags)
-        self._add_queue(queue)
+            queue = StandardQueue(queue_name, context.region, context.account_id, attributes, tags)
+
+        LOG.debug("creating queue key=%s attributes=%s tags=%s", queue_name, attributes, tags)
+
+        with self._mutex:
+            backend.queues[queue_name] = queue
 
         return CreateQueueResult(QueueUrl=queue.url(context))
 
     def get_queue_url(
         self, context: RequestContext, queue_name: String, queue_owner_aws_account_id: String = None
     ) -> GetQueueUrlResult:
-        account_id = queue_owner_aws_account_id or context.account_id
-        key = QueueKey(context.region, account_id, queue_name)
-
-        if key not in self.queues:
+        backend = SqsBackend.get(context.region)
+        if queue_name not in backend.queues.keys():
             raise QueueDoesNotExist("The specified queue does not exist for this wsdl version.")
 
-        queue = self.queues[key]
-        self._assert_permission(context, queue)
+        queue = backend.queues[queue_name]
 
         return GetQueueUrlResult(QueueUrl=queue.url(context))
 
@@ -757,17 +825,16 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         next_token: Token = None,
         max_results: BoxedInteger = None,
     ) -> ListQueuesResult:
-        urls = []
+        backend = SqsBackend.get(context.region)
 
-        for queue in self.queues.values():
-            if queue.key.region != context.region:
-                continue
-            if queue.key.account_id != context.account_id:
-                continue
-            if queue_name_prefix:
-                if not queue.name.startswith(queue_name_prefix):
-                    continue
-            urls.append(queue.url(context))
+        if queue_name_prefix:
+            urls = [
+                queue.url(context)
+                for queue in backend.queues.values()
+                if queue.name.startswith(queue_name_prefix)
+            ]
+        else:
+            urls = [queue.url(context) for queue in backend.queues.values()]
 
         if max_results:
             # FIXME: also need to solve pagination with stateful iterators: If the total number of items available is
@@ -786,7 +853,6 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         visibility_timeout: Integer,
     ) -> None:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
         queue.update_visibility_timeout(receipt_handle, visibility_timeout)
 
     def change_message_visibility_batch(
@@ -796,7 +862,6 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         entries: ChangeMessageVisibilityBatchRequestEntryList,
     ) -> ChangeMessageVisibilityBatchResult:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
 
         self._assert_batch(entries)
 
@@ -826,16 +891,16 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         )
 
     def delete_queue(self, context: RequestContext, queue_url: String) -> None:
+        backend = SqsBackend.get(context.region)
+
         with self._mutex:
             queue = self._resolve_queue(context, queue_url=queue_url)
-            self._assert_permission(context, queue)
-            del self.queues[queue.key]
+            del backend.queues[queue.name]
 
     def get_queue_attributes(
         self, context: RequestContext, queue_url: String, attribute_names: AttributeNameList = None
     ) -> GetQueueAttributesResult:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
 
         if not attribute_names:
             return GetQueueAttributesResult(Attributes={})
@@ -850,7 +915,7 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
             try:
                 getattr(QueueAttributeName, attr)
             except AttributeError:
-                raise InvalidAttributeName(f"Unknown attribute {attr}.")
+                raise InvalidAttributeName(f"Unknown Attribute {attr}.")
 
             if callable(queue.attributes.get(attr)):
                 func = queue.attributes.get(attr)
@@ -872,7 +937,6 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         message_group_id: String = None,
     ) -> SendMessageResult:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
 
         message = self._put_message(
             queue,
@@ -896,7 +960,6 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         self, context: RequestContext, queue_url: String, entries: SendMessageBatchRequestEntryList
     ) -> SendMessageBatchResult:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
 
         self._assert_batch(entries)
 
@@ -955,8 +1018,6 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         message_deduplication_id: String = None,
         message_group_id: String = None,
     ) -> Message:
-        # TODO: default message attributes (SenderId, ApproximateFirstReceiveTimestamp, ...)
-
         check_message_content(message_body)
         check_attributes(message_attributes)
         check_attributes(message_system_attributes)
@@ -1003,12 +1064,19 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         receive_request_attempt_id: String = None,
     ) -> ReceiveMessageResult:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
+
+        if wait_time_seconds is None:
+            wait_time_seconds = queue.wait_time_seconds
 
         num = max_number_of_messages or 1
-        block = wait_time_seconds is not None
+        block = True if wait_time_seconds else False
         # collect messages
         messages = []
+
+        # we chose to always return the maximum possible number of messages, even though AWS will typically return
+        # fewer messages than requested on small queues. at some point we could maybe change this to randomly sample
+        # between 1 and max_number_of_messages.
+        # see https://docs.aws.amazon.com/AWSSimpleQueueService/latest/APIReference/API_ReceiveMessage.html
         while num:
             try:
                 standard_message = queue.get(
@@ -1018,6 +1086,11 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
             except Empty:
                 break
 
+            # setting block to false guarantees that, if we've already waited before, we don't wait the full time
+            # again in the next iteration if max_number_of_messages is set but there are no more messages in the
+            # queue. see https://github.com/localstack/localstack/issues/5824
+            block = False
+
             moved_to_dlq = False
             if (
                 queue.attributes
@@ -1026,20 +1099,18 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
                 moved_to_dlq = self._dead_letter_check(queue, standard_message, context)
             if moved_to_dlq:
                 continue
-            # filter attributes
-            if message_attribute_names:
-                if "All" not in message_attribute_names:
-                    msg["MessageAttributes"] = {
-                        k: v
-                        for k, v in msg["MessageAttributes"].items()
-                        if k in message_attribute_names
-                    }
-                # TODO: why is this called even if we receive "All" attributes?
+
+            msg = copy.deepcopy(msg)
+            message_filter_attributes(msg, attribute_names)
+            message_filter_message_attributes(msg, message_attribute_names)
+
+            if msg.get("MessageAttributes"):
                 msg["MD5OfMessageAttributes"] = _create_message_attribute_hash(
                     msg["MessageAttributes"]
                 )
             else:
-                del msg["MessageAttributes"]
+                # delete the value that was computed when creating the message
+                msg.pop("MD5OfMessageAttributes")
 
             # add message to result
             messages.append(msg)
@@ -1056,10 +1127,14 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         max_receive_count = redrive_policy["maxReceiveCount"]
         if std_m.receive_times > max_receive_count:
             dead_letter_target_arn = redrive_policy["deadLetterTargetArn"]
-            dl_queue = self._require_queue_by_arn(dead_letter_target_arn)
+            dl_queue = self._require_queue_by_arn(context, dead_letter_target_arn)
             # TODO: this needs to be atomic?
             dead_message = std_m.message
-            dl_queue.put(message=dead_message)
+            dl_queue.put(
+                message=dead_message,
+                message_deduplication_id=std_m.message_deduplication_id,
+                message_group_id=std_m.message_group_id,
+            )
             queue.remove(std_m.message["ReceiptHandle"])
             return True
         else:
@@ -1069,7 +1144,6 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         self, context: RequestContext, queue_url: String, receipt_handle: String
     ) -> None:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
         queue.remove(receipt_handle)
 
     def delete_message_batch(
@@ -1079,7 +1153,6 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         entries: DeleteMessageBatchRequestEntryList,
     ) -> DeleteMessageBatchResult:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
         self._assert_batch(entries)
 
         successful = []
@@ -1107,7 +1180,6 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
 
     def purge_queue(self, context: RequestContext, queue_url: String) -> None:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
 
         with self._mutex:
             # FIXME: use queue-specific locks
@@ -1161,12 +1233,12 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
                 valid_count = False
             if not valid_count:
                 raise InvalidParameterValue(
-                    f"Value {redrive_policy} for parameter RedrivePolicy is invalid. Reason: Invalid value for maxReceiveCount: {max_receive_count}, valid values are from 1 to 1000 both inclusive."
+                    f"Value {redrive_policy} for parameter RedrivePolicy is invalid. Reason: Invalid value for "
+                    f"maxReceiveCount: {max_receive_count}, valid values are from 1 to 1000 both inclusive. "
                 )
 
     def tag_queue(self, context: RequestContext, queue_url: String, tags: TagMap) -> None:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
 
         if not tags:
             return
@@ -1176,12 +1248,10 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
 
     def list_queue_tags(self, context: RequestContext, queue_url: String) -> ListQueueTagsResult:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
         return ListQueueTagsResult(Tags=queue.tags)
 
     def untag_queue(self, context: RequestContext, queue_url: String, tag_keys: TagKeyList) -> None:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
 
         for k in tag_keys:
             if k in queue.tags:
@@ -1196,7 +1266,6 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         actions: ActionNameList,
     ) -> None:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
 
         self._validate_actions(actions)
 
@@ -1206,7 +1275,6 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
 
     def remove_permission(self, context: RequestContext, queue_url: String, label: String) -> None:
         queue = self._resolve_queue(context, queue_url=queue_url)
-        self._assert_permission(context, queue)
 
         candidates = [p for p in queue.permissions if p.label == label]
         if candidates:
@@ -1218,7 +1286,7 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         message_system_attributes: MessageBodySystemAttributeMap = None,
     ) -> Dict[MessageSystemAttributeName, str]:
         result: Dict[MessageSystemAttributeName, str] = {
-            MessageSystemAttributeName.SenderId: context.account_id,
+            MessageSystemAttributeName.SenderId: context.account_id,  # not the account ID in AWS
             MessageSystemAttributeName.SentTimestamp: str(now()),
         }
 
@@ -1249,23 +1317,6 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
                     "WSDL for a list of valid actions. "
                 )
 
-    def _assert_permission(self, context: RequestContext, queue: SqsQueue):
-        action = context.operation.name
-        account_id = context.account_id
-
-        if account_id == queue.owner:
-            return
-
-        for permission in queue.permissions:
-            if permission.account_id != account_id:
-                continue
-            if permission.action == "*":
-                return
-            if permission.action == action:
-                return
-
-        raise CommonServiceException("AccessDeniedException", "Not allowed")
-
     def _assert_batch(self, batch: List):
         if not batch:
             raise EmptyBatchRequest
@@ -1314,16 +1365,16 @@ def get_queue_name_from_url(queue_url: str) -> str:
     return queue_url.rstrip("/").split("/")[-1]
 
 
-def resolve_queue_key(
+def resolve_queue_name(
     context: RequestContext, queue_name: Optional[str] = None, queue_url: Optional[str] = None
-) -> QueueKey:
+) -> str:
     """
-    Resolves a QueueKey from the given information.
+    Resolves a queue name from the given information.
 
     :param context: the request context, used for getting region and account_id, and optionally the queue_url
     :param queue_name: the queue name (if this is set, then this will be used for the key)
     :param queue_url: the queue url (if name is not set, this will be used to determine the queue name)
-    :return: the QueueKey describing the queue being requested
+    :return: the queue name describing the queue being requested
     """
     if not queue_name:
         if queue_url:
@@ -1331,4 +1382,67 @@ def resolve_queue_key(
         else:
             queue_name = get_queue_name_from_url(context.request.base_url)
 
-    return QueueKey(region=context.region, account_id=context.account_id, name=queue_name)
+    return queue_name
+
+
+def message_filter_attributes(message: Message, names: Optional[AttributeNameList]):
+    """
+    Utility function filter from the given message (in-place) the system attributes from the given list. It will
+    apply all rules according to:
+    https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/sqs.html#SQS.Client.receive_message.
+
+    :param message: The message to filter (it will be modified)
+    :param names: the attributes names/filters
+    """
+    if "Attributes" not in message:
+        return
+
+    if not names:
+        del message["Attributes"]
+        return
+
+    if "All" in names:
+        return
+
+    for k in list(message["Attributes"].keys()):
+        if k not in names:
+            del message["Attributes"][k]
+
+
+def message_filter_message_attributes(message: Message, names: Optional[MessageAttributeNameList]):
+    """
+    Utility function filter from the given message (in-place) the message attributes from the given list. It will
+    apply all rules according to:
+    https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/sqs.html#SQS.Client.receive_message.
+
+    :param message: The message to filter (it will be modified)
+    :param names: the attributes names/filters (can be 'All', '.*', or prefix filters like 'Foo.*')
+    """
+    if not message.get("MessageAttributes"):
+        return
+
+    if not names:
+        del message["MessageAttributes"]
+        return
+
+    if "All" in names or ".*" in names:
+        return
+
+    attributes = message["MessageAttributes"]
+    matched = []
+
+    keys = [name for name in names if ".*" not in name]
+    prefixes = [name.split(".*")[0] for name in names if ".*" in name]
+
+    # match prefix filters
+    for k in attributes:
+        if k in keys:
+            matched.append(k)
+            continue
+
+        for prefix in prefixes:
+            if k.startswith(prefix):
+                matched.append(k)
+            break
+
+    message["MessageAttributes"] = {k: attributes[k] for k in matched}
