@@ -13,6 +13,7 @@ from typing import Dict, Iterable, List, Optional, Set
 from localstack import config, constants
 from localstack.config import Directories
 from localstack.runtime import hooks
+from localstack.utils.container_networking import get_main_container_name
 from localstack.utils.container_utils.container_client import (
     ContainerException,
     PortMappings,
@@ -106,33 +107,6 @@ def get_docker_image_details(image_name: str = None) -> Dict[str, str]:
     return result
 
 
-def get_main_container_ip():
-    container_name = get_main_container_name()
-    return DOCKER_CLIENT.get_container_ip(container_name)
-
-
-def get_main_container_id():
-    container_name = get_main_container_name()
-    try:
-        return DOCKER_CLIENT.get_container_id(container_name)
-    except ContainerException:
-        return None
-
-
-def get_main_container_name():
-    global MAIN_CONTAINER_NAME_CACHED
-    if MAIN_CONTAINER_NAME_CACHED is None:
-        hostname = os.environ.get("HOSTNAME")
-        if hostname:
-            try:
-                MAIN_CONTAINER_NAME_CACHED = DOCKER_CLIENT.get_container_name(hostname)
-            except ContainerException:
-                MAIN_CONTAINER_NAME_CACHED = config.MAIN_CONTAINER_NAME
-        else:
-            MAIN_CONTAINER_NAME_CACHED = config.MAIN_CONTAINER_NAME
-    return MAIN_CONTAINER_NAME_CACHED
-
-
 def get_image_environment_variable(env_name: str) -> Optional[str]:
     image_name = get_docker_image_to_start()
     image_info = DOCKER_CLIENT.inspect_image(image_name)
@@ -649,7 +623,7 @@ def configure_container(container: LocalstackContainer):
     container.env_vars["DOCKER_HOST"] = f"unix://{config.DOCKER_SOCK}"
     container.env_vars["HOST_TMP_FOLDER"] = config.dirs.functions  # TODO: rename env var
 
-    # TODO discuss if this should be the default?
+    # TODO this is default now, remove once a considerate time is passed
     # to activate proper signal handling
     container.env_vars["SET_TERM_HANDLER"] = "1"
 
